@@ -5,6 +5,8 @@ use warnings;
 
 use parent 'FusionInventory::Agent::Task::Inventory::Module';
 
+use English qw(-no_match_vars);
+
 use FusionInventory::Agent::Tools;
 
 sub isEnabled {
@@ -12,6 +14,9 @@ sub isEnabled {
 
     return if $params{no_category}->{user};
 
+    # Not working under win32
+    return 0 if $OSNAME eq 'MSWin32';
+
     return
         canRun('who')  ||
         canRun('last') ||
