@@ -3,6 +3,8 @@ package FusionInventory::Agent::Task::Inventory::Win32::Modems;
 use strict;
 use warnings;
 
+use parent 'FusionInventory::Agent::Task::Inventory::Module';
+
 use FusionInventory::Agent::Tools::Win32;
 
 sub isEnabled {
@@ -11,6 +13,12 @@ sub isEnabled {
     return 1;
 }
 
+sub isEnabledForRemote {
+    my (%params) = @_;
+    return 0 if $params{no_category}->{modem};
+    return 1;
+}
+
 sub doInventory {
     my (%params) = @_;
 
