@@ -19,6 +19,7 @@ use FusionInventory::Agent::Tools::Network;
 use FusionInventory::Agent::Tools::Expiration;
 
 use FusionInventory::Agent::Task::NetInventory::Version;
+use FusionInventory::Agent::Task::NetInventory::Job;
 
 our $VERSION = FusionInventory::Agent::Task::NetInventory::Version::VERSION;
 
@@ -61,11 +62,14 @@ sub isEnabled {
             next;
         }
 
-        push @jobs, {
-            params      => $option->{PARAM}->[0],
+        my $params = $option->{PARAM}->[0];
+
+        push @jobs, FusionInventory::Agent::Task::NetInventory::Job->new(
+            logger      => $self->{logger},
+            params      => $params,
             credentials => $option->{AUTHENTICATION},
             devices     => \@devices
-        };
+        );
     }
 
     if (!@jobs) {
@@ -78,6 +82,56 @@ sub isEnabled {
     return 1;
 }
 
+sub _inventory_thread {
+    my ($self, $jobs, $done) = @_;
+
+    my $id = threads->tid();
+    $self->{logger}->debug("[thread $id] creation");
+
+    # run as long as they are a job to process
+    while (my $job = $jobs->dequeue()) {
+
+        last unless ref($job) eq 'HASH';
+        last if $job->{leave};
+
+        my $device = $job->{device};
+
+        my $result;
+        eval {
+            $result = $self->_queryDevice($job);
+        };
+        if ($EVAL_ERROR) {
+            chomp $EVAL_ERROR;
+            $result = {
+                ERROR => {
+                    ID      => $device->{ID},
+                    MESSAGE => $EVAL_ERROR
+                }
+            };
+
+            $result->{ERROR}->{TYPE} = $device->{TYPE} if $device->{TYPE};
+
+            # Inserted back device PID in result if set by server
+            $result->{PID} = $device->{PID} if defined($device->{PID});
+
+            $self->{logger}->error("[thread $id] $EVAL_ERROR");
+        }
+
+        # Get result PID from result
+        my $pid = delete $result->{PID};
+
+        # Directly send the result message from the thread, but use job pid if
+        # it was not set in result
+        $self->_sendResultMessage($result, $pid || $job->{pid});
+
+        $done->enqueue($job);
+    }
+
+    delete $self->{logger}->{prefix};
+
+    $self->{logger}->debug("[thread $id] termination");
+}
+
 sub run {
     my ($self, %params) = @_;
 
@@ -93,105 +147,134 @@ sub run {
         no_compress  => $params{no_compress},
     ) if !$self->{client};
 
+    # Extract greatest max_threads from jobs
+    my ($max_threads) = sort { $b <=> $a } map { int($_->max_threads()) }
+        @{$self->{jobs}};
+
+    my %running_threads = ();
+
+    # initialize FIFOs
+    my $jobs = Thread::Queue->new();
+    my $done = Thread::Queue->new();
+
+    # count devices and check skip_start_stop
+    my $devices_count   = 0;
+    my $skip_start_stop = 0;
+    foreach my $job (@{$self->{jobs}}) {
+        $devices_count += $job->count();
+        # newer server won't need START message if PID is provided on <DEVICE/>
+        $skip_start_stop = any { defined($_->{PID}) } $job->devices()
+            unless $skip_start_stop;
+    }
+
+    # Define a job expiration: 15 minutes by device to scan is large enough
+    setExpirationTime( timeout => $devices_count * 900 );
+    my $expiration = getExpirationTime();
+
+    # no need more threads than devices to scan
+    my $threads_count = $max_threads > $devices_count ? $devices_count : $max_threads;
+
+    $self->{logger}->debug("creating $threads_count worker threads");
+    for (my $i = 0; $i < $threads_count; $i++) {
+        my $newthread = threads->create(sub { $self->_inventory_thread($jobs, $done); });
+        # Keep known created threads in a hash
+        $running_threads{$newthread->tid()} = $newthread ;
+        usleep(50000) until ($newthread->is_running() || $newthread->is_joinable());
+    }
+
+    # Check really started threads number vs really running ones
+    my @really_running  = map { $_->tid() } threads->list(threads::running);
+    my @started_threads = keys(%running_threads);
+    unless (@really_running == $threads_count && keys(%running_threads) == $threads_count) {
+        $self->{logger}->debug(scalar(@really_running)." really running: [@really_running]");
+        $self->{logger}->debug(scalar(@started_threads)." started: [@started_threads]");
+    }
+
+    my %queues = ();
+    my $pid_index = 1;
+
+    # Start jobs by preparing queues
     foreach my $job (@{$self->{jobs}}) {
-        my $max_threads = $job->{params}->{THREADS_QUERY};
-        my $timeout     = $job->{params}->{TIMEOUT};
 
         # SNMP credentials
-        my $credentials = _getIndexedCredentials($job->{credentials});
+        my $credentials = $job->credentials();
 
-        # set internal state
-        $self->{pid} = $job->{params}->{PID} || '';
-
-        # newer server won't need START message if PID is provided on <DEVICE/>
-        my $skip_start_stop = any { defined($_->{PID}) } @{$job->{devices}};
+        # set pid
+        my $pid = $job->pid() || $pid_index++;
 
         # send initial message to server unless it supports newer protocol
-        $self->_sendStartMessage() unless $skip_start_stop;
+        $self->_sendStartMessage($pid) unless $skip_start_stop;
 
-        my ($debug_sent_count, $started_count) = ( 0, 0 );
-        my %running_threads = ();
-
-        # initialize FIFOs
-        my $devices = Thread::Queue->new();
-        my $results = Thread::Queue->new();
-
-        foreach my $device (@{$job->{devices}}) {
-            $devices->enqueue($device);
-        }
-        my $size = $devices->pending();
-
-        # no need for more threads than devices to scan
-        my $threads_count = $max_threads > $size ? $size : $max_threads;
-
-        # Define a job expiration: 15 minutes by device to scan is large enough
-        setExpirationTime(
-            timeout => $size * (!$timeout || $timeout < 900 ?  900 : $timeout)
-        );
-        my $expiration = getExpirationTime();
-
-        my $sub = sub {
-            my $id = threads->tid();
-            $self->{logger}->debug("[thread $id] creation");
-
-            # run as long as they are devices to process
-            while (my $device = $devices->dequeue_nb()) {
-
-                my $result;
-                eval {
-                    $result = $self->_queryDevice(
-                        device      => $device,
-                        timeout     => $timeout,
-                        credentials => $credentials->{$device->{AUTHSNMP_ID}}
-                    );
-                };
-                if ($EVAL_ERROR) {
-                    chomp $EVAL_ERROR;
-                    $result = {
-                        ERROR => {
-                            ID      => $device->{ID},
-                            TYPE    => $device->{TYPE},
-                            MESSAGE => $EVAL_ERROR
-                        }
-                    };
-
-                    # Inserted back device PID in result if set by server
-                    $result->{PID} = $device->{PID} if defined($device->{PID});
-
-                    $self->{logger}->error("[thread $id] $EVAL_ERROR");
-                }
-
-                $results->enqueue($result) if $result;
-            }
-
-            $self->{logger}->debug("[thread $id] termination");
+        # prepare queue
+        my $queue = $queues{$pid} || {
+            max_in_queue    => $job->max_threads(),
+            in_queue        => 0,
+            todo            => []
         };
-
-        $self->{logger}->debug("creating $threads_count worker threads");
-        for (my $i = 0; $i < $threads_count; $i++) {
-            my $newthread = threads->create($sub);
-            # Keep known created threads in a hash
-            $running_threads{$newthread->tid()} = $newthread ;
-            usleep(50000) until ($newthread->is_running() || $newthread->is_joinable());
+        foreach my $device ($job->devices()) {
+            push @{$queue->{todo}}, {
+                pid         => $pid,
+                device      => $device,
+                timeout     => $job->timeout(),
+                credentials => $credentials->{$device->{AUTHSNMP_ID}}
+            };
         }
 
-        # Check really started threads number vs really running ones
-        my @really_running  = map { $_->tid() } threads->list(threads::running);
-        my @started_threads = keys(%running_threads);
-        unless (@really_running == $threads_count && keys(%running_threads) == $threads_count) {
-            $self->{logger}->debug(scalar(@really_running)." really running: [@really_running]");
-            $self->{logger}->debug(scalar(@started_threads)." started: [@started_threads]");
+        # Only keep queue if we have a device to scan
+        $queues{$pid} = $queue
+            if @{$queue->{todo}};
+    }
+
+    my $queued_count = 0;
+    my $job_count = 0;
+    my $jid_len = length(sprintf("%i",$devices_count));
+    my $jid_pattern = "#%0".$jid_len."i";
+
+    # We need to guaranty we don't have more than max_in_queue device in shared
+    # queue for each job
+    while (my @pids = sort { $a <=> $b } keys(%queues)) {
+
+        # Enqueue as device as possible
+        foreach my $pid (@pids) {
+            my $queue = $queues{$pid};
+            next unless @{$queue->{todo}};
+            next if $queue->{in_queue} >= $queue->{max_in_queue};
+            my $device = shift @{$queue->{todo}};
+            $queue->{in_queue} ++;
+            $device->{jid} = sprintf($jid_pattern, ++$job_count);
+            $jobs->enqueue($device);
+            $queued_count++;
         }
-        $started_count += @started_threads ;
 
         # as long as some of our threads are still running...
-        while (keys(%running_threads)) {
+        if (keys(%running_threads)) {
 
             # send available results on the fly
-            while (my $result = $results->dequeue_nb()) {
-                $self->_sendResultMessage($result);
+            while (my $device = $done->dequeue_nb()) {
+                my $pid = $device->{pid};
+                my $queue = $queues{$pid};
+                $queue->{in_queue} --;
+                $queued_count--;
+                unless ($queue->{in_queue} || @{$queue->{todo}}) {
+                    # send final message to the server before cleaning threads unless it supports newer protocol
+                    $self->_sendStopMessage($pid) unless $skip_start_stop;
+
+                    delete $queues{$pid};
+
+                    # send final message to the server unless it supports newer protocol
+                    $self->_sendStopMessage($pid) unless $skip_start_stop;
+                }
+                # Check if it's time to abort a thread
+                $devices_count--;
+                if ($devices_count < $threads_count) {
+                    $jobs->enqueue({ leave => 1 });
+                    $threads_count--;
+                }
             }
 
+            # wait for a little
+            usleep(50000);
+
             if ($expiration && time > $expiration) {
                 $self->{logger}->warning("Aborting netinventory job as it reached expiration time");
                 # detach all our running worker
@@ -203,9 +286,6 @@ sub run {
                 last;
             }
 
-            # wait for a little
-            usleep(50000);
-
             # List our created and possibly running threads in a list to check
             my %running_threads_checklist = map { $_ => 0 }
                 keys(%running_threads);
@@ -224,27 +304,21 @@ sub run {
                 delete $running_threads{$tid}
                     unless $running_threads_checklist{$tid};
             }
+            last unless keys(%running_threads);
         }
-
-        # Reset expiration
-        setExpirationTime();
-
-        # purge remaining results
-        while (my $result = $results->dequeue_nb()) {
-            $self->_sendResultMessage($result);
-        }
-
-        # send final message to the server before cleaning threads unless it supports newer protocol
-        $self->_sendStopMessage() unless $skip_start_stop;
-
-        if ($started_count) {
-            $self->{logger}->debug("cleaning $started_count worker threads");
-            $_->join() foreach threads->list(threads::joinable);
-        }
-
-        # send final message to the server unless it supports newer protocol
-        $self->_sendStopMessage() unless $skip_start_stop;
     }
+
+    if ($queued_count) {
+        $self->{logger}->error("$queued_count devices inventory are missing");
+    }
+
+    # Cleanup joinable threads
+    $_->join() foreach threads->list(threads::joinable);
+    $self->{logger}->debug("All netinventory threads terminated")
+        unless threads->list(threads::running);
+
+    # Reset expiration
+    setExpirationTime();
 }
 
 sub _sendMessage {
@@ -264,7 +338,7 @@ sub _sendMessage {
 }
 
 sub _sendStartMessage {
-    my ($self) = @_;
+    my ($self, $pid) = @_;
 
     $self->_sendMessage({
         AGENT => {
@@ -272,48 +346,42 @@ sub _sendStartMessage {
             AGENTVERSION => $FusionInventory::Agent::Version::VERSION,
         },
         MODULEVERSION => $VERSION,
-        PROCESSNUMBER => $self->{pid}
+        PROCESSNUMBER => $pid
     });
 }
 
 sub _sendStopMessage {
-    my ($self) = @_;
+    my ($self, $pid) = @_;
 
     $self->_sendMessage({
         AGENT => {
             END => 1,
         },
         MODULEVERSION => $VERSION,
-        PROCESSNUMBER => $self->{pid}
-    });
-}
-
-sub _sendResultMessage {
-    my ($self, $result) = @_;
-
-    my $pid = $self->{pid};
-    if (exists($result->{PID})) {
-        # Don't keep PID in result but we want to set it as parent PROCESSNUMBER
-        $pid = $result->{PID} if defined($result->{PID});
-        delete $result->{PID};
-    }
-
-    $self->_sendMessage({
-        DEVICE        => $result,
-        MODULEVERSION => $VERSION,
         PROCESSNUMBER => $pid
     });
 }
 
-sub _queryDevice {
-    my ($self, %params) = @_;
+sub _sendResultMessage {
+    my ($self, $result, $pid) = @_;
 
-    my $credentials = $params{credentials};
-    my $device      = $params{device};
+    $self->_sendMessage({
+        DEVICE        => $result,
+        MODULEVERSION => $VERSION,
+        PROCESSNUMBER => $pid || 0
+    });
+}
+
+sub _queryDevice {
+    my ($self, $params) = @_;
+
+    my $credentials = $params->{credentials};
+    my $device      = $params->{device};
     my $logger      = $self->{logger};
     my $id          = threads->tid();
+    $logger->{prefix} = "[thread $id] $params->{jid}, ";
     $logger->debug(
-        "[thread $id] scanning $device->{ID}: $device->{IP}" .
+        "scanning $device->{ID}: $device->{IP}" .
         ( $device->{PORT} ? ' on port ' . $device->{PORT} : '' ) .
         ( $device->{PROTOCOL} ? ' via ' . $device->{PROTOCOL} : '' )
     );
@@ -336,7 +404,7 @@ sub _queryDevice {
                 hostname     => $device->{IP},
                 port         => $device->{PORT},
                 domain       => $device->{PROTOCOL},
-                timeout      => $params{timeout} || 15,
+                timeout      => $params->{timeout} || 15,
                 community    => $credentials->{COMMUNITY},
                 username     => $credentials->{USERNAME},
                 authpassword => $credentials->{AUTHPASSPHRASE},
@@ -352,7 +420,7 @@ sub _queryDevice {
          id      => $device->{ID},
          type    => $device->{TYPE},
          snmp    => $snmp,
-         model   => $params{model},
+         model   => $params->{model},
          logger  => $self->{logger},
          datadir => $self->{datadir}
     );
@@ -363,13 +431,6 @@ sub _queryDevice {
     return $result;
 }
 
-sub _getIndexedCredentials {
-    my ($credentials) = @_;
-
-    # index credentials by their ID
-    return { map { $_->{ID} => $_ } @{$credentials} };
-}
-
 1;
 
 __END__
