@@ -3,10 +3,10 @@ package FusionInventory::Agent::Tools::Screen::Samsung;
 use strict;
 use warnings;
 
-use base 'FusionInventory::Agent::Tools::Screen';
+use parent 'FusionInventory::Agent::Tools::Screen';
 
 # Well-known eisa_id for which wee need to revert serial and altserial
-my $eisa_id_match = qr/09c6$/ ;
+my $eisa_id_match = qr/09c6|0ca5$/ ;
 
 sub serial {
     my ($self) = @_;
