@@ -787,6 +787,9 @@ sub _isVirtual {
 
     # PhysicalAdapter only work on OS > XP
     if (defined $object->{PhysicalAdapter}) {
+        # Some virtual network adapters like VirtualBox or VPN ones could be set
+        # as physical but with PNPDeviceID starting by ROOT
+        return 1 if (defined($object->{PNPDeviceID}) && $object->{PNPDeviceID} =~ /^ROOT/);
         return $object->{PhysicalAdapter} ? 0 : 1;
     }
 
@@ -872,7 +875,7 @@ sub getAgentMemorySize {
     };
     return -1 unless (defined($ph));
 
-    my $size = -1;
+    my ($size, $pages) = ( -1, 0 );
     eval {
         # memory usage is bundled up in ProcessMemoryCounters structure
         # populated by GetProcessMemoryInfo() win32 call
@@ -907,8 +910,9 @@ sub getAgentMemorySize {
             )'
         );
         if ($apiGetProcessMemoryInfo->Call($ph, $mem_counters, $cb)) {
-            # Uses WorkingSetSize as process memory size
+            # Uses WorkingSetSize and PagefileUsage
             $size = $mem_counters->{WorkingSetSize};
+            $pages = $mem_counters->{PagefileUsage};
         }
     };
 
@@ -923,7 +927,7 @@ sub getAgentMemorySize {
         $ph = $apiCloseHandle->Call($ph);
     };
 
-    return $size;
+    return $size, $pages;
 }
 
 sub FreeAgentMem {
@@ -963,6 +967,10 @@ my @win32_ole_calls : shared;
 sub start_Win32_OLE_Worker {
 
     unless (defined($worker)) {
+
+        # Handle thread KILL signal
+        $SIG{KILL} = sub { threads->exit(); };
+
         # Request a semaphore on which worker blocks immediatly
         Thread::Semaphore->require();
         $worker_semaphore = Thread::Semaphore->new(0);
@@ -970,6 +978,8 @@ sub start_Win32_OLE_Worker {
         # Start a worker thread
         $worker = threads->create( \&_win32_ole_worker );
     }
+
+    return $worker;
 }
 
 sub setupWorkerLogger {
